;;;; -*- Mode: Lisp -*-
;;;; $Id: usocket-scl.lisp 592 2008-10-28 12:10:59Z binghe $

;;;; UDP support for Scieneer CL, commit by Douglas Crosher.
;;;; Note: this code only supports the SCL after 1.3.8.1.

(in-package :usocket)

(defmethod socket-close :after ((socket datagram-usocket))
  (setf (%open-p socket) nil))

(defun socket-connect/udp (host port &key element-type timeout deadline nodelay
			   (local-host nil local-host-p)
			   (local-port nil local-port-p)
			   &aux
			   (patch-udp-p (fboundp 'ext::inet-socket-send-to)))
  (declare (ignore element-type timeout deadline nodelay))

  (when (and local-host-p (not patch-udp-p))
     (unsupported 'local-host 'socket-connect :minimum "1.3.8.2"))
  (when (and local-port-p (not patch-udp-p))
     (unsupported 'local-port 'socket-connect :minimum "1.3.8.2"))

  (let ((socket))
    (when (not patch-udp-p)
      (error 'unsupported
	     :feature '(protocol :datagram)
	     :context 'socket-connect
	     :minumum "1.3.8.2 or ask a udp-patch from SCL maintainer"))
    (setf socket
	  (if (and host port)
	      (let ((args (list (host-to-hbo host) port :kind :datagram)))
		(when (and patch-udp-p (or local-host-p local-port-p))
		  (nconc args (list :local-host (when local-host
						  (host-to-hbo local-host))
				    :local-port local-port)))
		(with-mapped-conditions (socket)
		  (apply #'ext:connect-to-inet-socket args)))
	      (if (or local-host-p local-port-p)
		  (with-mapped-conditions ()
		    (ext:create-inet-listener (or local-port 0)
					      :datagram
					      :host (when local-host
						      (if (ip= local-host *wildcard-host*)
							  0
							  (host-to-hbo local-host)))))
		  (with-mapped-conditions ()
		    (ext:create-inet-socket :datagram)))))
    (let ((usocket (make-datagram-socket socket)))
      (ext:finalize usocket #'(lambda ()
				(when (%open-p usocket)
				  (ext:close-socket socket))))
      usocket)))

(defmethod socket-send ((socket datagram-usocket) buffer length &key host port)
  (let ((s (socket socket))
	(host (if host (host-to-hbo host))))
    (multiple-value-bind (result errno)
	(ext:inet-socket-send-to s buffer length
				 :remote-host host :remote-port port)
      (or result
	  (scl-map-socket-error errno :socket socket)))))

(defmethod socket-receive ((socket datagram-usocket) buffer length)
  (let ((s (socket socket)))
    (let ((real-buffer (or buffer
			   (make-array length :element-type '(unsigned-byte 8))))
	  (real-length (or length
			   (length buffer))))
      (multiple-value-bind (result errno remote-host remote-port)
	  (ext:inet-socket-receive-from s real-buffer real-length)
	(if result
	    (values real-buffer result remote-host remote-port)
	    (scl-map-socket-error errno :socket socket))))))
