;;;; -*- Mode: Lisp -*-
;;;; $Id: usocket.lisp 625 2008-11-15 16:58:25Z binghe $

(in-package :usocket)

(export '(datagram-usocket
          socket-connect/udp
          with-udp-client-socket
          socket-send
          socket-receive
          socket-sync
          socket-server
          socket-warning ; condition
          *remote-host*
          *remote-port*))

(defparameter *major-version* 2)
(defparameter *minor-version* 4)

(eval-when (:load-toplevel :execute)
  (unless (fboundp 'wait-for-input)
    (error "Please use a newer USOCKET package (>= 0.4.0).")))

(defconstant +max-datagram-packet-size+ 65536)

;; redefine datagram-usocket, add rtt-info-mixin
(defclass datagram-usocket (usocket rtt-info-mixin)
  ((connected-p :type boolean
                :accessor connected-p
                :initarg :connected-p)
   #+(or cmu scl lispworks)
   (%open-p     :type boolean
                :accessor %open-p
                :initform t
		:documentation "Flag to indicate if usocket is open,
for GC on implementions operate on raw socket fd."))
  (:documentation "UDP (inet-datagram) socket"))

(defun datagram-usocket-p (socket)
  (typep socket 'datagram-usocket))

(defun make-datagram-socket (socket &key connected-p)
  (unless socket
    (error 'invalid-socket-error))
  (make-instance 'datagram-usocket
                 :socket socket
                 :connected-p connected-p))

(defmacro with-udp-client-socket ((socket-var &rest socket-connect-args)
                                  &body body)
  "Bind the socket resulting from a call to `socket-connect/udp' with
the arguments `socket-connect-args' to `socket-var' and if `stream-var' is
non-nil, bind the associated socket stream to it."
  `(with-connected-socket (,socket-var
			   (socket-connect/udp ,@socket-connect-args))
     (progn ,@body)))
